#ifndef _COORD_H
#define _COORD_H

#include <glibmm/ustring.h>
#include <string>
#include <iostream>
#include "Error.h"

using Glib::ustring;
using namespace std;

//! Coordinates on the goban.
/*!
 * Convenience class to pack the coordinates on the goban.
 */
class Coord {
    private:
        int m_i;
        int m_j;
    public:
        //! Get the first coordinate.
        /*!
         * \return the first coordinate
         */
        int i() const {return m_i;}
        //! Get the second coordinate.
        /*!
         * \return the second coordinate
         */
        int j() const {return m_j;}
        //! Default constructor
        /*!
         * Sets the coordinates to (-1,-1)
         */
        Coord():m_j(-1),m_i(-1) {}
        //! Constructor
        /*!
         * \param c a string repersenting coordinates.
         * The coordinates can be specified by a couple of lowcase letters a-z.
         * 'a' maps to '0', 'b' to '1' and so on.
         * This is the coordinates format used in the 'Smart Game Format' (SGF)
         * supported by this program.
         * \throw CoordError if the string was badly formatted.
         */
        Coord(const ustring& c);
        //! Constructor
        /*!
         * \param ai the first coordinate
         * \param aj the second coordinate
         */
        Coord(int ai, int aj): m_i(ai), m_j(aj) {}
        //! Copy constructor
        /*!
         * \param c the Coord to copy.
         */
        Coord(const Coord& c): m_i(c.i()), m_j(c.j()) {}
        //! Affectation operator.
        /*!
         * \param c the Coord to affect to this.
         */
        Coord& operator=(const Coord& c);
        //! Conversion operator to a string in the SGF format.
        /*!
         * \param c the to be converted to a string.
         * \see Coord(const ustring& c)
         */
        operator ustring () const;
        //! Comparison operator.
        /*!
         * \param left a Coord to compare
         * \param right a Coord to compare
         * \return true if both coordinates i() and j() are identical.
         */
        friend bool operator==(const Coord& left, const Coord& right);
        friend bool operator!=(const Coord& left, const Coord& right) {return !operator==(left,right);}
        friend std::ostream& operator<<(std::ostream& out, const Coord& right);
};
//! The Error class for the Coord class.
class CoordError: public Error{
    public:
        CoordError(const ustring& err):Error(err) {}
        friend ostream& operator<<(ostream& os, const CoordError& err);
};

//! A label on the goban
/*!
 * A convenience class to represent labels on the goban.
 */
class Label {
    public:
        //! Supported types of labels
        enum Type {
            CIRCLE,//!< A circle
            TRIANGLE,//!< A triangle
            SQUARE,//!< A square
            LETTER//!< A letter
        };

        //! Constructor
        /*!
         * \param coord the Coord of the label.
         * \param type the type of label.
         * \param letter if type is Label::LETTER, the letter label.
         */
        Label(const Coord& coord,Label::Type type, char letter=0): m_coord(coord),m_type(type),m_letter(letter) {}
        //! Get the coordinates.
        /*!
         * \return the coordinates.
         * \see Coord
         */
        const Coord& Coordinates() const {return m_coord;}
        //! Get the type of label.
        /*!
         * \return the type.
         * \see Label::Type
         */
        Type GetType() const {return (Type)m_type;}
        //! Get the letter label.
        /*!
         * Does not check whether it is applicable
         * (whether the type is Label::LETTER).
         * \return the letter of the label.
         */
        const char& Letter() const {return m_letter;}

    private:
        Coord m_coord;
        Type m_type;
        char m_letter;
};

#endif /*_COORD_H*/
