#ifndef _GOBANLOGIC_H
#define _GOBANLOGIC_H

#include <utility>
#include "Logger.h"
#include "SgfNode.h"
//! Class implementing the go rules and keeping track of the SGF game tree.
/*!
 * This is the main interface between the GUI and the internals. The GobanLogic class
 * keeps track of the root node of the game tree, the current node being played, and
 * the modifications of the game tree. It also implements the go game rules.
 * \todo Add Ko detection and mark it.
 * \todo When loading a game tree, the parser only checks the syntaxic correctness.
 * Semantic errors are only detected when actually attempting to play the bad node, resulting
 * in an exception raised. Maybe a file correctness should be checked semantically as well, but
 * that imply playing the whole game tree (so there is a significant slowing up when opening a file.)
 */
class GobanLogic {
    private:
        //! The root node
        SgfNode* m_sgf_root;
        //! The current node
        SgfNode* m_sgf_current;
        //! Color being played
        bool m_black;
        //! Is the game tree modified since last save?
        bool m_modified;
        bool m_last_passed;
        double m_komi;

        //! Get a group of stones (in the go game meaning)
        void get_group(const Coord& c, list<Coord>& group, list<Coord>& lib);
        //! Interprete the properties of a SgfNode. The semantic analysis takes place here.
        void treat_node(SgfNode* node);
        //! Treat an attempted move.
        /*
         * \return true if the node was legal, false otherwise.
         */
        bool treat_move(Board& board, const Coord& c, bool black);// Capture-scuicide rules

    public:
        //!Constructor
        /*!
         * \param sgfstream the SGF tree game to load (using SgfNode::BuildTree).
         * If none is given, an empty tree is created.
         */
        GobanLogic(const ustring& sgfstream="");
        //!Destructor
        /*!
         * The tree game is owned by the GobanLogic instance. It is deleted by it.
         */
        ~GobanLogic();
        //!Play the game tree starting from a given node.
        /*!
         * All the variations are played.
         * \param node the node to start from.
         * \param errors an array of erroneus nodes (illegal moves).
         */
        void PlayGameTree(SgfNode* node, list<pair<SgfNode*,ustring> > & errors);
        //!Erase the game tree boards from a given node.
        /*!
         * All variations are reset. Useful when one need to recompute the final positions
         * when a change has been made at the beginning. Note that illegal position may arize
         * when doing so.
         * \param node the node to start from
         */
        void FreeGameTree(SgfNode* node);
        //!Create the final node with the final status storred (dead stones, territories)
        const Board* SetFinalState(vector<Coord> dead, vector<Coord> black_terr, vector<Coord> white_terr);
        //!Add a stone on the board.
        /*!
         * The go rules are not applied in this case. Used to setup
         * a go problem for instance. It will not create a node, but just
         * add a SgfProperty::AB or SgfProperty::AW property to the current node.
         * \param black the color of the stone.
         * \param coord the coordinates where to put the stone
         * \return the board with the added stone.
         */
        const Board* AddStone(const Coord& coord, bool black);
        //!Remove a stone from the board.
        /*!
         * The go rules are not observed. Will only work on stones
         * previously added with the GobanLogic::AddStone method
         * in the same node (That is it will remove the corresponding
         * value of the SgfProperty::AB or SgfProperty::AW property.)
         * If those conditions are not met, an exception is raised.
         * \param coord the coordinates of the stone to remove.
         * \return the modified board.
         */
        const Board* RemoveStone(const Coord& coord);
        double GetKomi() const {return m_komi;}
        void SetKomi(double komi) {m_komi=komi;}
        //!Get current game node
        const SgfNode* CurrentNode() const {return m_sgf_current;}
        //!Get current game node
        SgfNode* CurrentNode() {return m_sgf_current;}
        //!Get the root node
        const SgfNode* RootNode() const {return m_sgf_root;}
        //!Get last node of a given branch
        SgfNode* LastNode(SgfNode* node);
        //!Get last node of a given branch
        const SgfNode* LastNode(const SgfNode* node) const;
        //!Get the branching node the node originates
        const SgfNode* PreviousBranchingNode(const SgfNode* st) const;
        SgfNode* PreviousBranchingNode(SgfNode* st);
        const SgfNode* NextBranchingNode(const SgfNode* st) const;
        SgfNode* NextBranchingNode(SgfNode* st);
        //!Get the root node
        SgfNode* RootNode() {return m_sgf_root;}
        //!Play a node
        /*!
         * Play a node and return a (const) pointer to the resulting Board.
         * The nodes coming before 'node' will be recursively played in order, from
         * the succession of moves, to determine the board state of the current node.
         * \param node a pointer to the node to play.
         * \return the resulting (and allocated, see the Board doc) Board.
         * \throw GobanLogicError if the root node is missing essential information (e.g. size of the board).
         * \throw GobanLogicError if the node contains an illegal move.
         * \throw SgfPropertyError if a node property value is badly formatted
         */
        const Board* PlayNode(SgfNode* node);
        //!Resignation
        void Resign();
        //!Get the current node score defined in its origin embranchement
        ustring GetScore();
        //!Gets the node just before the node where the first move is defined.
        /*!
         * If no move is found, returns the last node of the first variation.
         */
        SgfNode* NodeBeforeFirstMove();
        //! Tells if a following node is defined in the game tree.
        bool HasNextNode() const {if(m_sgf_current->NChilds()) return true; else return false;}
        //! Play the next node.
        /*!
         * Play the first variation coming after this node, that is its first child node.
         * \return The board state (allocated, see Board doc)
         * \throw GobanLogicError if there is no following node in the game tree.
         */
        const Board* PlayNextNode();
        //! Tells if there is a preceding node (always true except for the root node)
        bool HasPreviousNode() const {if(m_sgf_current->Parent()) return true; else return false;}
        //! Play the preceding (parent) node
        /*!
         * \return the preceding node board state
         * \throw GobanLogicError if the current node is the root node (has no parent)
         */
        const Board* PlayPreviousNode();
        //! Tells whether there is a alternative coming after the current node.
        /*!
         * \return true if there is a following adjacent node.
         * \see SgfNode::HasNextBranch
         */
        bool HasNextBranch() const
            {
                if(m_sgf_current->Parent() && m_sgf_current->Parent()->HasNextBranch(m_sgf_current))
                    return true;
                else
                    return false;
            }
        //! Play the following variation to the current node
        /*!
         * \return the variation board state
         * \throw SgfError if there is no next branch
         */
        const Board* PlayNextBranch() {return PlayNode(m_sgf_current->Parent()->NextBranch(m_sgf_current));}
        //! Tells whether there is a alternative coming before the current node.
        /*!
         * \return true if there is a preceding adjacent node
         * \see SgfNode::HasPreviousBranch
         */
        bool HasPreviousBranch() const
            {
                if(m_sgf_current->Parent() && m_sgf_current->Parent()->HasPreviousBranch(m_sgf_current))
                    return true;
                else
                    return false;
            }
        //! Play the preceding variation to the current node
        /*!
         * \return the variation board state
         * \throw SgfError if there is no previous branch
         */
        const Board* PlayPreviousBranch() { return PlayNode(m_sgf_current->Parent()->PreviousBranch(m_sgf_current));}
        //! Play a move
        /*!
         * If the move already exsists in the game tree, simply play the corresponding node (see SgfNode::PlayNode)
         * and set it as the current node.
         * Otherwise, add a new node to the game tree. The parent node is the current node.
         * Then set the current node to the new one.
         * \param c The coordinates of the move.
         * \return the new board state
         * \throw GobanLogicError if the move is illegal
         * \throw SgfPropertyError if the move corresponds to an exsisting node which has a property value badly formatted.
         */
        const Board* PlayMove(const Coord& c);
        //! Tells if it is to player black to play.
        bool Black() const {return m_black;}
        //! Set the player which has to play the next node.
        bool SetBlack(bool black) {m_black=black;}
        //! Tells if the game tree has been modified since last save.
        bool Modified() {return m_modified;}
        //! Set the game tree as modified.
        void SetModified(bool m) {m_modified=m;}
};

//!The GobanLogic error class.
class GobanLogicError: public Error{
    public:
        GobanLogicError(const ustring& err, bool fatal=false, int code=0):Error(err,fatal,code) {}
        friend ostream& operator<<(ostream& os, const GobanLogicError& err);
};

#endif /*_GOBANLOGIC_H*/
