#ifndef _SGFNODE_H
#define _SGFNODE_H
#include <vector>
#include <stack>
#include <algorithm>
#include <exception>
#include "SgfProperty.h"
#include "Board.h"

typedef Glib::ustring::value_type uchar;

//!A node of the game tree
/*!
 * Base class of the SGF parser. Each node has a parent node except the root node, and a
 * list of child nodes. It also has a list of SgfProperties, and a Board which is not
 * allocated at first. The Board state is allocated by the GobanLogic class by recursively playing all the nodes
 * from the root node.
 */
class SgfNode {
    private:
        //! The list of properties
        list<SgfProperty*> m_prop;
        // The list of childs
        vector<SgfNode*> m_childs;
        //! the parent node
        SgfNode* m_parent;
        //! the board
        Board m_board;
        //!final node
        bool m_final;

        //! Gets the 'CA' property in a file in order to know the charset encoding.
        string get_sgf_charset(const string& in);

    public:
        //!Constructor
        /*!
         * \param the parent node
         */
        SgfNode(SgfNode* parent=0): m_parent(parent), m_final(false) {if(parent) parent->AddChild(this);}
        //!Destructor
        /*!
         * Recursively delete all its children
         */
        ~SgfNode();
        //!Gets an iterator to the beginning of the property list.
        list<SgfProperty*>::const_iterator Begin() const {return m_prop.begin();}
        //!Gets an iterator to the beginning of the property list.
        list<SgfProperty*>::iterator Begin() {return m_prop.begin();}
        //!Gets an iterator to the end of the property list.
        list<SgfProperty*>::const_iterator End() const {return m_prop.end();}
        //!Gets an iterator to the end of the property list.
        list<SgfProperty*>::iterator End() {return m_prop.end();}
        //!Add a property to the node.
        /*!
         * \param prop a pointer to the property. The SgfNode will own the property
         * (it will be deleted by the SgfNode destructor)
         */
        void AddProperty(SgfProperty* prop) {m_prop.push_back(prop);}
        //!Add a property to the node.
        /*!
         * \param id the property id
         * \param value the property value
         */
        void AddProperty(SgfProperty::SgfPropEnum id, ustring value) {AddProperty(new SgfProperty(id,value));}
        //!Remove a property from the node.
        /*!
         * \param prop a pointer to the property
         * \todo throw a n exception if the property is not found.
         */
        void RemoveProperty(SgfProperty* prop);
        //!Gets the number of child nodes.
        unsigned int NChilds() const {return m_childs.size();}
        void AddBlackCaptures(int capt) {m_board.AddBlackCaptures(capt);}
        void AddWhiteCaptures(int capt) {m_board.AddWhiteCaptures(capt);}
        //!Gets the n'th child.
        /*!
         * \param n the child index.
         * \throw std::exception if n is out of range
         */
        const SgfNode* ChildAt(unsigned int n) const {return m_childs.at(n);}
        //!Gets the n'th child.
        /*!
         * \param n the child index.
         * \throw std::exception if n is out of range
         */
        SgfNode* ChildAt(unsigned int n) {return m_childs.at(n);}
        //!Tells if the parent node has a child node coming after this one.
        bool HasNextBranch(const SgfNode* child) const;
        //!Gets the child of the parent node which comes after this one.
        /*!
         * \throw SgfError if there is no next branch.
         */
        const SgfNode* NextBranch(const SgfNode* child) const;
        //!Gets the child of the parent node which comes after this one.
        /*!
         * \throw SgfError if there is no next branch.
         */
        SgfNode* NextBranch(const SgfNode* child);
        //!Tells if the parent node has a child node comes before this one.
        bool HasPreviousBranch(const SgfNode* child) const;
        //!Gets the child of the parent node which comes before this one.
        /*!
         * \throw SgfError if there is no previous branch.
         */
        const SgfNode* PreviousBranch(const SgfNode* child) const;
        //!Gets the child of the parent node which comes before this one.
        /*!
         * \throw SgfError if there is no previous branch.
         */
        SgfNode* PreviousBranch(const SgfNode* child);
        //!Add a child node.
        /*!
         * The child nods are owned by their parent.
         * \param child the child node to add.
         */
        void AddChild(SgfNode* child) {m_childs.push_back(child);}
        //!Remove a child node
        /*!
         * \param child a pointer to the child node.
         * \todo throw an exception if the child id not found.
         */
        void RemoveChild(SgfNode* child);
        //!Gets the parent node
        const SgfNode* Parent() const {return m_parent;}
        //!Gets the parent node
        SgfNode* Parent() {return m_parent;}
        //! Gets the board.
        const Board& GetBoard() const {return m_board;}
        //! Gets the board.
        Board& GetBoard() {return m_board;}
        bool GetFinal() const {return m_final;}
        void SetFinal(bool final) {m_final=final;}
        //! Gets a property from its id
        /*
         * \param id the id of the property
         * \return the property with the corresponding id or 0 if none found.
         */
        const SgfProperty* Property(const SgfProperty::SgfPropEnum& ident) const;
        //! Gets a property from its id
        /*
         * \param id the id of the property
         * \return the property with the corresponding id or 0 if none found.
         */
        SgfProperty* Property(const SgfProperty::SgfPropEnum& ident);
        //!Gets the node in its SGF string representation
        /*!
         * All the childs nodes SGF string representations will be recursively appended.
         * \return The SGF string representation of the node.
         */
        string str();
        //!Build a game tree from an SGF string.
        /*!
         * This function parse the input string and builds a game tree.
         * \param in the SGF string
         * \return a pointer to the root node.
         * \throw SgfError if the string is badly formatted.
         */
        static SgfNode* BuildTree(const ustring& in);
        //!Output stream operator
        friend ostream& operator<<(ostream & out,const SgfNode& node);
};

//!The SgfNode error class.
class SgfError: public Error{
    protected:
        const SgfNode* m_node;
    public:
        SgfError(ustring err, const SgfNode* node):Error(err),m_node(node){}
        const SgfNode* error_node() const {return m_node;}
        friend ostream& operator<<(ostream& out,const SgfError& err);
};

#endif /*_SGFNODE_H*/
