#ifndef _SGFPROPERTY_H
#define _SGFPROPERTY_H
#include <map>
#include <string>
#include <list>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <glibmm.h>
#include "Coord.h"

using namespace std;
using Glib::ustring;
//! A class to represent an sgf property
/*!
 * This class implements a property of the 'Smart Game Format' (SGF) used to store go games.
 * Each property has an id taken from the list of symbols defined in the enum SgfProperty::SgfPropEnum
 * and a list of values. In an SGF file, a property is written 'XX[...][...][...]...' where 'XX' is the
 * property id and the strings in the brackets the values.
 * For more informations on the SGF format see http://www.red-bean.com/sgf/
 */
class SgfProperty {
    public:
        //! Definition of all the id symbols part of FF[4]
        /*!
         * \see SgfPropEnum.def
         */
        enum SgfPropEnum{
        //include the FF[4] properties declarations.
        #include "SgfPropEnum.def"
        };
        //!Constructor
        /*!
         * \param id the id of the property.
         * \param value a value for the property.
         */
        SgfProperty(SgfProperty::SgfPropEnum id=SgfProperty::SgfPropEnum(-1), ustring value="");
        //!Set the property id
        /*!
         * \param ident a string identifying the id. If the string does not match an id listed in
         * SgfProperty::SgfPropEnum, the string id is set nontheless but the property will be
         * ignored by the parser.
         */
        void SetPropertyIdent(const ustring& ident);
        //!Set the property id
        /*!
         * \param ident the property ident
         */
        void SetPropertyIdent(const SgfPropEnum& ident);
        //!Set the n'th value
        /*!
         * \param n the index of the value to set.
         * \param val the value string.
         * \throw SgfPropertyError if n is out of bounds.
         */
        void SetValueAt(int n, ustring val, bool form=false);
        //!Add a value to the property
        /*!
         * \param content the content string
         */
        void AddValue(const ustring& content, bool form=false);
        //!Remove a value from the property.
        /*!
         * \param value the value to remove.
         * \throw SgfPropertyError if the value is not found.
         * \see SgfProperty::HasValue
         */
        void RemoveValue(const ustring& value);
        //! Tells whether 'value' is in the list of values of the SgfProperty.
        bool HasValue(const ustring & value) const;
        //!Gets the number of values.
        int NContent() const {return m_prop_value.size();}
        //!Gets an iterator to the beginning of the values list.
        list<ustring>::const_iterator Begin() const {return m_prop_value.begin();}
        //!Gets an iterator to the end of the values list.
        list<ustring>::const_iterator End() const {return m_prop_value.end();}
        //!Gets an iterator to the beginning of the values list
        list<ustring>::iterator Begin() {return m_prop_value.begin();}
        //!Gets an iterator to the end of the values list.
        list<ustring>::iterator End() {return m_prop_value.end();}
        //!Gets the Property id.
        SgfPropEnum PropIdent() const {return (SgfPropEnum)m_prop_ident;}
        //!Gets a value as a string.
        /*!
         * \param i the index of the value
         * \param composed if the property value is composed, tells whether the first part or the composed part
         * of the value is wanted. Ignored if the property is not composed.
         * \return the i'th value as a string
         * \throw SgfPropertyError if the index is out of bounds.
         */
        ustring Value2U(const int& i=0, bool composed=false) const {return Value2U(prop_value_at(i), composed);}
        //!Gets a value as a string.
        /*!
         * \param iter an iterator to the value.
         * \param composed if the property value is composed, tells whether the first part or the composed part
         * of the value is wanted. Ignored if the property is not composed.
         * \return the value at iter as a string
         * \throw SgfPropertyError if the iterator points to SgfProperty::End().
         */
        ustring Value2U(list<ustring>::const_iterator iter, bool composed) const;
        //!Gets a value as an int.
        /*!
         * \param i the index of the value
         * \param composed if the property value is composed, tells whether the first part or the composed part
         * of the value is wanted. Ignored if the property is not composed.
         * \return the i'th value as an int
         * \throw SgfPropertyError if the index is out of bounds.
         * \throw SgfPropertyError if the value string does not represent an integer.
         */
        int Value2I(const int& i=0, bool composed=false) const {return Value2I(prop_value_at(i),composed);}
        //!Gets a value as an int.
        /*!
         * \param iter an iterator to the value.
         * \param composed if the property value is composed, tells whether the first part or the composed part
         * of the value is wanted. Ignored if the property is not composed.
         * \return the value at iter as an int
         * \throw SgfPropertyError if the iterator points to SgfProperty::End().
         * \throw SgfPropertyError if the value string does not represent an integer.
         */
        int Value2I(list<ustring>::const_iterator iter, bool composed=false) const;
        //!Gets a value as a double.
        /*!
         * \param i the index of the value
         * \param composed if the property value is composed, tells whether the first part or the composed part
         * of the value is wanted. Ignored if the property is not composed.
         * \return the i'th value as double
         * \throw SgfPropertyError if the index is out of bounds.
         * \throw SgfPropertyError if the value string does not represent a double.
         */
        double Value2D(const int& i=0, bool composed=false) const {return Value2D(prop_value_at(i),composed);}
        //!Gets a value as a double.
        /*!
         * \param iter an iterator to the value.
         * \param composed if the property value is composed, tells whether the first part or the composed part
         * of the value is wanted. Ignored if the property is not composed.
         * \return the value at iter as a double
         * \throw SgfPropertyError if the iterator points to SgfProperty::End().
         * \throw SgfPropertyError if the value string does not represent an double.
         */
        double Value2D(list<ustring>::const_iterator iter, bool composed=false) const;
        //!Gets a value as a Coord.
        /*!
         * \param i the index of the value
         * \param composed if the property value is composed, tells whether the first part or the composed part
         * of the value is wanted. Ignored if the property is not composed.
         * \return the i'th value a Coord
         * \throw SgfPropertyError if the index is out of bounds.
         * \throw CoordError if the value string does not represent a Coord.
         */
        Coord Value2C(const int& i=0, bool composed=false) const {return Value2C(prop_value_at(i), composed);}
        //!Gets a value as a Coord.
        /*!
         * \param iter an iterator to the value.
         * \param composed if the property value is composed, tells whether the first part or the composed part
         * of the value is wanted. Ignored if the property is not composed.
         * \return the value at iter as a Coord
         * \throw SgfPropertyError if the iterator points to SgfProperty::End().
         * \throw CoordError if the value string does not represent an Coord.
         */
        Coord Value2C(list<ustring>::const_iterator iter, bool composed=false) const;
        //! Output stream operator.
        friend ostream& operator<<(ostream& out, const SgfProperty & sgfprop);

    private:
        //! a map mapping id strings to SgfProperty::SgfPropEnum ids
        static map<ustring,SgfPropEnum> m_map_str2e;
        //! a map mapping SgfProperty::SgfPropEnum ids to id strings
        static map<SgfPropEnum,ustring> m_map_e2str;
        //!the id
        int m_prop_ident;
        //!whether the property might be composed or not
        bool m_composed;
        //!the string id
        ustring m_prop_str_ident;
        //!the list of values
        list<ustring> m_prop_value;

        //!Add a '\\' to characters which should not be interpreted the parser.
        static ustring add_escape_char(ustring in,ustring chars);
        //!Remove the '\\' to characters which should not be interpreted the parser.
        static ustring remove_escape_char(ustring in);
        //!Tells whether the value of the property id is formatted text
        static bool is_formatted_text(int id);
        //!Split the value at the semicolon.
        void split_composed(const ustring& in, ustring& out1, ustring& out2) const;
        //!Gets an iterator to the i'th value. const method
        list<ustring>::const_iterator prop_value_at(const int& i) const;
        //!Gets an iterator to the i'th value.
        list<ustring>::iterator prop_value_at(const int& i);
        //!Build the maps m_map_str2e and m_map_e2str (shared resource)
        static void BuildPropDict();
        //!Tells if the property id 'id' might be composed.
        bool might_be_composed(SgfProperty::SgfPropEnum id);
};

//! The SgfProperty Error class
class SgfPropertyError: public Error{
    public:
        SgfPropertyError(ustring err):Error(err) {}
        friend ostream& operator<<(ostream& os, const SgfPropertyError& err);
};

#endif /*_SGFPROPERTY_H*/
